@@ -187,8 +187,8 @@ void alarm_resume( unsigned boot ){
 			resume_alarm = 1; // alarm_close() needs to relock tk
 			hw_unlock_screen();
 			hw_unlock_keys();
+			if(hw_screen.locked || cfg_tkactive_notifications) notify_screen_unlocked();
 			hw_screen.locked = 0;
-			notify_screen_unlocked();
 		}
 
 		alarm_open(tklock,
