@@ -1,6 +1,8 @@
 //	Advanced System UI
 //	Copyright (c) 2010-2011 Brand Huntsman <brand.huntsman@gmail.com>
 
+#include <unistd.h>
+
 #include "window.h"
 #include "main.h"
 #include "draw.h"
@@ -65,6 +67,7 @@ void click_flight_mode_button( ){
 
 			// enter flight mode and stop all related services
 			mce_flight_mode();
+			usleep(100000);
 			svc_init_gps(SVC_STOP);
 			svc_init_bluetooth(SVC_STOP, 0);
 			svc_init_wifi(SVC_STOP, 0);
