@@ -26,6 +26,7 @@ const char *conf_screen_stayslit2 = "/system/osso/dsm/display/display_on_with_ch
 const char *conf_brightness_level = "/apps/asui_state/brightness_level";
 const char *conf_theme = "/apps/asui_state/theme";
 const char *conf_ssh_server = "/apps/asui_state/ssh_server";
+const char *conf_battery_usage = "/apps/asui_state/battery_usage";
 
 const char *conf_alarm_open = "/apps/asui_state/alarm_open";
 const char *conf_alarm_tklock = "/apps/asui_state/alarm_tklock";
@@ -41,7 +42,7 @@ const char *conf_alarm_timestamp = "/apps/asui_state/alarm_timestamp";
 GConfClient *gconf_client;
 
 // battery
-unsigned cfg_battery_idle_time, cfg_memory_format, cfg_swap_format, cfg_temperature_F;
+unsigned cfg_battery_idle_time, cfg_memory_format, cfg_swap_format, cfg_temperature_F, cfg_history_days, cfg_track_usage_events;
 // clock
 unsigned cfg_clock_24h, cfg_clock_ampm;
 unsigned cfg_popup_duration, cfg_popup_onmap, cfg_popup_fullscreen, cfg_popup_cooldown, cfg_popup_noasui;
@@ -55,12 +56,12 @@ unsigned cfg_buttons_dpad_short, cfg_buttons_dpad_long, cfg_buttons_lock1_short,
 unsigned cfg_secure_action, cfg_secure_power_action, cfg_secure_button;
 unsigned cfg_power_unlock_key;
 // user interface
-unsigned cfg_service_size_unit, cfg_short_tap_markers, cfg_start_visible, cfg_threshold_black, cfg_threshold_white;
+unsigned cfg_tkactive_notifications, cfg_service_size_unit, cfg_short_tap_markers, cfg_start_visible, cfg_threshold_black, cfg_threshold_white;
 // alarm
 unsigned cfg_alarm_duration, cfg_alarm_volume_min, cfg_alarm_volume_max, cfg_alarm_volume_step;
 
 // state
-unsigned cfg_theme, cfg_ssh_server;
+unsigned cfg_theme, cfg_ssh_server, cfg_battery_usage;
 
 ////////////////////////////////////////////////////////////////////////// WIDGETS
 
@@ -106,7 +107,9 @@ static const char *conf_battery_idle_time = "/apps/asui/battery_idle_time";
 static const char *conf_memory_format = "/apps/asui/memory_format";
 static const char *conf_swap_format = "/apps/asui/swap_format";
 static const char *conf_temperature_F = "/apps/asui/temperature_F";
-static const char *conf_enable_battery_applet = "/apps/asui/enable_battery_applet";
+static const char *conf_history_days = "/apps/asui/history_days";
+static const char *conf_track_usage_events = "/apps/asui/track_usage_events";
+static const char *conf_enable_battery_applet = "/apps/asui/enable_battery_applet"; // not used by ASUI
 // clock
 static const char *conf_clock_24h = "/apps/asui/clock_24h";
 static const char *conf_clock_ampm = "/apps/asui/clock_ampm";
@@ -140,6 +143,7 @@ static const char *conf_secure_power_action = "/apps/asui/secure_power_action";
 static const char *conf_secure_button = "/apps/asui/secure_button";
 static const char *conf_power_unlock_key = "/apps/asui/power_unlock_key";
 // user interface
+static const char *conf_tkactive_notifications = "/apps/asui/tkactive_notifications";
 static const char *conf_service_size_unit = "/apps/asui/service_size_unit";
 static const char *conf_short_tap_markers = "/apps/asui/short_tap_markers";
 static const char *conf_start_visible = "/apps/asui/start_visible";
@@ -349,6 +353,8 @@ void config_init( ){
 	gconf_client_unset(gconf_client, conf_memory_format, NULL);
 	gconf_client_unset(gconf_client, conf_swap_format, NULL);
 	gconf_client_unset(gconf_client, conf_temperature_F, NULL);
+	gconf_client_unset(gconf_client, conf_history_days, NULL);
+	gconf_client_unset(gconf_client, conf_track_usage_events, NULL);
     gconf_client_unset(gconf_client, conf_enable_battery_applet, NULL);
 	// clock
 	gconf_client_unset(gconf_client, conf_clock_24h, NULL);
@@ -383,6 +389,7 @@ void config_init( ){
 	gconf_client_unset(gconf_client, conf_secure_button, NULL);
 	gconf_client_unset(gconf_client, conf_power_unlock_key, NULL);
 	// user interface
+	gconf_client_unset(gconf_client, conf_tkactive_notifications, NULL);
 	gconf_client_unset(gconf_client, conf_service_size_unit, NULL);
 	gconf_client_unset(gconf_client, conf_short_tap_markers, NULL);
 	gconf_client_unset(gconf_client, conf_start_visible, NULL);
@@ -425,6 +432,7 @@ void config_init( ){
 	gconf_client_unset(gconf_client, conf_brightness_level, NULL);
 	gconf_client_unset(gconf_client, conf_theme, NULL);
 	gconf_client_unset(gconf_client, conf_ssh_server, NULL);
+	gconf_client_unset(gconf_client, conf_battery_usage, NULL);
 	gconf_client_unset(gconf_client, "/apps/asui_state", NULL);
 
 	printf("all gconf keys cleared\n");
@@ -458,6 +466,36 @@ void config_init( ){
 	}
 }
 
+#ifdef ARCH_armel
+static void free_usage_events( ){
+	s_event_history *history;
+
+	// free all screen history elements
+	for(history = hw_screen.history_head; history != NULL;){
+		s_event_history *next = history->next;
+		free(history);
+		history = next;
+	}
+	hw_screen.history_head = hw_screen.history_tail = NULL;
+
+	// free all wifi history elements
+	for(history = hw_wifi.history_head; history != NULL;){
+		s_event_history *next = history->next;
+		free(history);
+		history = next;
+	}
+	hw_wifi.history_head = hw_wifi.history_tail = NULL;
+
+	// free all bluetooth history elements
+	for(history = hw_bluetooth.history_head; history != NULL;){
+		s_event_history *next = history->next;
+		free(history);
+		history = next;
+	}
+	hw_bluetooth.history_head = hw_bluetooth.history_tail = NULL;
+}
+#endif
+
 void config_reload( ){
 	#ifdef ARCH_armel
 	// battery
@@ -465,6 +503,8 @@ void config_reload( ){
 	make_uint(&cfg_memory_format,			conf_memory_format,				3,		HW_BATTERY);		// default: 3	used/total
 	make_uint(&cfg_swap_format,				conf_swap_format,				1,		HW_BATTERY);		// default: 1	% used
 	make_bool(&cfg_temperature_F,			conf_temperature_F,				0,		HW_TEMPERATURE);	// default: 0	celsius
+	make_uint(&cfg_history_days,			conf_history_days,				7,		0);					// default: 7	7 days
+	make_bool(&cfg_track_usage_events, 		conf_track_usage_events,		0,		0);					// default: 0	off
 	(void)conf_enable_battery_applet;
 	// clock
 	make_bool(&cfg_clock_24h,				conf_clock_24h,					1,		0);					// default: 1	24 hour clock
@@ -499,6 +539,7 @@ void config_reload( ){
 	make_bool(&cfg_secure_button,			conf_secure_button,				0,		HW_THEME);			// default: 0	don't show secure button
 	make_uint(&cfg_power_unlock_key,		conf_power_unlock_key,		(is_810 ? 1:4),	0);				// default: 1/4	power button + fullscreen/dpad key unlocks
 	// user interface
+	make_bool(&cfg_tkactive_notifications,	conf_tkactive_notifications,	1,		0);					// default: 1	show screen on unlock notifications
 	make_bool(&cfg_service_size_unit,		conf_service_size_unit,			1,		HW_THEME);			// default: 1	show MB with service sizes
 	make_bool(&cfg_short_tap_markers,		conf_short_tap_markers,			0,		HW_THEME);			// default: 0	don't show short tap markers
 	make_bool(&cfg_start_visible,			conf_start_visible,				0,		0);					// default: 0	not visible
@@ -506,20 +547,44 @@ void config_reload( ){
 	make_uint(&cfg_threshold_white,			conf_threshold_white,			4000,	HW_THEME);			// default: 	to white theme when 4000 lux or greater
 	// alarm
 	make_uint(&cfg_alarm_duration,			conf_alarm_duration,			60,		0);					// default:		20-3600 seconds
-	make_uint(&cfg_alarm_volume_min,		conf_alarm_volume_min,			50,		0);					// default:		10-100
-	make_uint(&cfg_alarm_volume_max,		conf_alarm_volume_max,			100,	0);					// default:		10-100
-	make_uint(&cfg_alarm_volume_step,		conf_alarm_volume_step,			125,	0);					// default:		0-1000 milliseconds
+	make_uint(&cfg_alarm_volume_min,		conf_alarm_volume_min,			50,		0);					// default:		10-100 / 100
+	make_uint(&cfg_alarm_volume_max,		conf_alarm_volume_max,			100,	0);					// default:		10-100 / 100
+	make_uint(&cfg_alarm_volume_step,		conf_alarm_volume_step,			125,	0);					// default:		0-1000 / 1000
 	// state
 	unsigned brightness_level;
 	make_uint(&brightness_level,			conf_brightness_level,			31,		HW_BRIGHTNESS);		// default: 31	25% brightness
 	unsigned theme = !is_810;
 	make_uint(&cfg_theme,					conf_theme,						theme,	HW_THEME);			// default: 0/1	color theme
 	make_uint(&cfg_ssh_server,				conf_ssh_server,				0,		HW_BRIGHTNESS);		// default: 0	openssh
+	make_bool(&cfg_battery_usage,			conf_battery_usage,				0,		HW_BATTERY);		// default: 0	show active/idle times
 
 	// memory and swap formats can be 0-3
 	if(cfg_memory_format > 3){ cfg_memory_format = 0; conf_set_int(conf_memory_format, 0); }
 	if(cfg_swap_format > 3){ cfg_swap_format = 0; conf_set_int(conf_swap_format, 0); }
 
+	// battery history days
+	if(cfg_history_days > 7){ cfg_history_days = 7; conf_set_int(conf_history_days, 7); }
+	if(cfg_history_days == 0){
+		if(cfg_battery_usage) hw_changes |= HW_BATTERY;
+		cfg_battery_usage = 0;
+		if(hw_battery.history_head){
+			// free all battery history elements
+			s_battery_history *history;
+			for(history = hw_battery.history_head; history != NULL;){
+				s_battery_history *next = history->next;
+				free(history);
+				history = next;
+			}
+			// reset values in case history is turned back on
+			hw_battery.drain1 = hw_battery.drain24 = 0.0;
+			hw_battery.hours1 = 0;
+			hw_battery.hours24 = 2;
+			hw_battery.history_head = hw_battery.history_tail = hw_battery.charge_point = NULL;
+		}
+		free_usage_events();
+	}
+	if(!cfg_track_usage_events) free_usage_events();
+
 	// popup window duration
 	if(cfg_popup_duration > 15){ cfg_popup_duration = 5; conf_set_int(conf_popup_duration, 5); }
 	if(cfg_popup_cooldown < 60 || cfg_popup_cooldown > 3600){ cfg_popup_cooldown = 300; conf_set_int(conf_popup_cooldown, 300); }
@@ -601,12 +666,14 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 
 	#else
 
-	cfg_start_visible = 1; // no power button to map it
+	cfg_history_days = 0;
+	cfg_track_usage_events = 0;
+
 	cfg_clock_24h = 1; // because I like 24 hour
 	cfg_popup_duration = 0;
 	cfg_popup_onmap = 0;
-	cfg_threshold_white = 4000; // must be set otherwise white theme is used
-	cfg_short_tap_markers = 1; // show all markers
+	cfg_popup_noasui = 1;
+
 	cfg_double_tap = 1; // require double tap
 	cfg_buttons_dpad_short = 1;
 	cfg_buttons_dpad_long = 4;
@@ -618,8 +685,13 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	cfg_secure_power_action = 1;
 	cfg_secure_button = 1; // disable secure buttons
 
+	cfg_short_tap_markers = 1; // show all markers
+	cfg_start_visible = 1; // no power button to map it
+	cfg_threshold_white = 4000; // must be set otherwise white theme is used
+
 	cfg_theme = 1; // no ALS sensor, use blank theme
 	cfg_ssh_server = 0; // openssh
+	cfg_battery_usage = 0; // show active/idle times
 
 	#endif
 
@@ -631,6 +703,8 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	debug_write("  cfg_memory_format = %u", cfg_memory_format);
 	debug_write("  cfg_swap_format = %u", cfg_swap_format);
 	debug_write("  cfg_temperature_F = %u", cfg_temperature_F);
+	debug_write("  cfg_history_days = %u", cfg_history_days);
+	debug_write("  cfg_track_usage_events = %u", cfg_track_usage_events);
 	debug_write("  cfg_enable_battery_applet = %u", conf_get_bool(conf_enable_battery_applet));
 	// clock
 	debug_write("  cfg_clock_24h = %u", cfg_clock_24h);
@@ -665,6 +739,7 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	debug_write("  cfg_secure_button = %u", cfg_secure_button);
 	debug_write("  cfg_power_unlock_key = %u", cfg_power_unlock_key);
 	// user interface
+	debug_write("  cfg_tkactive_notifications = %u", cfg_tkactive_notifications);
 	debug_write("  cfg_service_size_unit = %u", cfg_service_size_unit);
 	debug_write("  cfg_short_tap_markers = %u", cfg_short_tap_markers);
 	debug_write("  cfg_start_visible = %u", cfg_start_visible);
@@ -681,6 +756,7 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	debug_write("  brightness_level = %u", brightness_level);
 	debug_write("  cfg_theme = %u", cfg_theme);
 	debug_write("  cfg_ssh_server = %u", cfg_ssh_server);
+	debug_write("  cfg_battery_usage = %u", cfg_battery_usage);
 	debug_write("------------------------------");
 }
 
