@@ -13,6 +13,7 @@ extern const char *conf_screen_stayslit2;
 extern const char *conf_brightness_level;
 extern const char *conf_theme;
 extern const char *conf_ssh_server;
+extern const char *conf_battery_usage;
 
 extern const char *conf_alarm_open;
 extern const char *conf_alarm_tklock;
@@ -28,7 +29,7 @@ extern const char *conf_alarm_timestamp;
 extern GConfClient *gconf_client;
 
 // battery
-extern unsigned cfg_battery_idle_time, cfg_memory_format, cfg_swap_format, cfg_temperature_F;
+extern unsigned cfg_battery_idle_time, cfg_memory_format, cfg_swap_format, cfg_temperature_F, cfg_history_days, cfg_track_usage_events;
 // clock
 extern unsigned cfg_clock_24h, cfg_clock_ampm;
 extern unsigned cfg_popup_duration, cfg_popup_onmap, cfg_popup_fullscreen, cfg_popup_cooldown, cfg_popup_noasui;
@@ -42,12 +43,12 @@ extern unsigned cfg_buttons_dpad_short, cfg_buttons_dpad_long, cfg_buttons_lock1
 extern unsigned cfg_secure_action, cfg_secure_power_action, cfg_secure_button;
 extern unsigned cfg_power_unlock_key;
 // user interface
-extern unsigned cfg_service_size_unit, cfg_short_tap_markers, cfg_start_visible, cfg_threshold_black, cfg_threshold_white;
+extern unsigned cfg_tkactive_notifications, cfg_service_size_unit, cfg_short_tap_markers, cfg_start_visible, cfg_threshold_black, cfg_threshold_white;
 // alarm
 extern unsigned cfg_alarm_duration, cfg_alarm_volume_min, cfg_alarm_volume_max, cfg_alarm_volume_step;
 
 // state
-extern unsigned cfg_theme, cfg_ssh_server;
+extern unsigned cfg_theme, cfg_ssh_server, cfg_battery_usage;
 
 #define CFG_BUTTONS_ZOOM 1
 #define CFG_BUTTONS_UD 2
