@@ -26,6 +26,7 @@
 #include "config.h"
 #include "services.h"
  #include "hardware.h"
+#include "dialog.h"
 #include "dbus.h"
 #include "dsme.h"
 #include "secure.h"
@@ -42,7 +43,7 @@ s_hw_memory hw_memory;
 s_hw_swap hw_swap;
 
 s_hw_screen hw_screen;
-unsigned hw_flight_mode;
+unsigned hw_flight_mode, hw_entering_flight_mode;
 unsigned hw_device_locked;
 
 s_hw_battery hw_battery;
@@ -77,6 +78,36 @@ int get_file_value( const char *filename ){
 	return value;
 }
 
+void hw_new_event( const char *label, s_event_history **head, s_event_history **tail, unsigned state ){
+	if(cfg_history_days == 0 || !cfg_track_usage_events) return;
+
+// TODO: this could test for a head element that will be purged below and reuse its memory instead of allocating here and freeing below
+	s_event_history *history = (s_event_history *)malloc(sizeof(s_event_history));
+	if(history){
+		history->timestamp = current_time;
+		history->state = state;
+		history->next = NULL;
+
+		if(*head == NULL)
+			*head = history;
+		else
+			(*tail)->next = history;
+		*tail = history;
+	} else
+		debug_write("[event] Unable to allocate memory to store %s history data point", label);
+
+	double base_time = current_time - cfg_history_days*86400;
+	for(history = *head; history != NULL;){
+		s_event_history *next = history->next;
+		if(history->timestamp > base_time) break;
+		// history element is older than storable time, remove it
+		if(*head == history) *head = next;
+		if(next == NULL) *tail = NULL;
+		free(history);
+		history = next;
+	}
+}
+
 ////////////////////////////////////////////////////////////////////////// HAL
 
 #ifdef ARCH_armel
@@ -238,7 +269,7 @@ static const char *_mce_get_device_mode_ = "get_device_mode"; // method
 static const char *_mce_req_device_mode_change_ = "req_device_mode_change"; // method
 static const char *_mce_device_NORMAL_ = "normal";
 static const char *_mce_device_FLIGHT_ = "flight";
-static unsigned got_flight_mode, sl_ready;
+static unsigned sl_ready;
 
 static const char *_mce_devlock_callback_ = "devlock_callback"; // method
 
@@ -271,6 +302,7 @@ static void mce_get_display_callback( DBusPendingCall *pending, void *user_data
 	if(dbus_message_get_args(message, &dbus_error, _STRING_, &mode, _END_)){
 		debug_write("[DBUS] callback: mce_get_display_callback (mode = %s)", mode);
 		hw_screen.visible = (strcmp(mode, "off") ? 1 : 0);
+		hw_new_event("screen", &hw_screen.history_head, &hw_screen.history_tail, hw_screen.visible);
 		if(sl_ready) sl_init(); else sl_ready++;
 		unsigned dimmed = (strcmp(mode, "dimmed") ? 0 : 1);
 		if(hw_screen.dimmed != dimmed){
@@ -314,7 +346,6 @@ static void mce_get_flight_callback( DBusPendingCall *pending, void *user_data )
 	if(dbus_message_get_args(message, &dbus_error, _STRING_, &mode, _END_)){
 		unsigned flight = (strcmp(mode, _mce_device_NORMAL_) ? 1 : 0);
 		if(hw_flight_mode != flight){ hw_flight_mode = flight; hw_changes |= HW_FLIGHT_MODE; }
-		got_flight_mode = 1;
 		hw_initialized &= ~HWIS_MCE_FLIGHT;
 	}
 	dbus_message_unref(message);
@@ -329,8 +360,11 @@ void mce_init( ){
 	hw_screen.visible_time = 0.0;
 	hw_screen.locked = 0;
 	hw_flight_mode = 0;
+	hw_entering_flight_mode = 0;
 	hw_device_locked = 0;
 
+	hw_screen.history_head = hw_screen.history_tail = NULL;
+
 	#ifdef ARCH_armel
 	sl_ready = 0;
 	#endif
@@ -490,40 +524,26 @@ void hw_unlock_keys( ){
 
 ////////////////////
 
-static void hw_query_flight_mode( unsigned flush ){
-	// called from hw_refresh(), mce_normal_mode() and mce_flight_mode()
-
-	#ifdef ARCH_armel
-	got_flight_mode = 0;
-	dbus_recv_method(mce_get_flight_callback, _mce_service_, _mce_path_, _mce_interface_, _mce_get_device_mode_, _END_);
-	if(flush){
-		dbus_flush_messages();
-		int i; // wait 2 seconds for reply
-		for(i = 0; i < 20; i++){
-			usleep(100000); // 100ms sleep to let MCE process method
-			if(got_flight_mode) break;
-		}
-	}
-	#endif
-}
-
 void mce_normal_mode( ){
 	#ifdef ARCH_armel
 	// enable normal mode
 	dbus_send_method(_mce_service_, _mce_path_, _mce_interface_, _mce_req_device_mode_change_, _STRING_, &_mce_device_NORMAL_, _END_);
-
-	usleep(100000); // 100ms sleep to let MCE process method
-	hw_query_flight_mode(1);
 	#endif
 }
 
 void mce_flight_mode( ){
 	#ifdef ARCH_armel
-	// enable flight mode
-	dbus_send_method(_mce_service_, _mce_path_, _mce_interface_, _mce_req_device_mode_change_, _STRING_, &_mce_device_FLIGHT_, _END_);
+	hw_entering_flight_mode = hw_wifi.enabled + hw_bluetooth.enabled;
+	// disconnect wifi so ICD won't run in flight mode
+	if(hw_wifi.enabled) hw_wifi_disconnect();
+	// turn off bluetooth in case it has similar problems
+	if(hw_bluetooth.enabled) hw_set_bluetooth(0);
 
-	usleep(100000); // 100ms sleep to let MCE process method
-	hw_query_flight_mode(1);
+	// enable flight mode
+	if(!hw_entering_flight_mode){
+		usleep(100000); // sleep 100ms to be sure services have fully disconnected
+		dbus_send_method(_mce_service_, _mce_path_, _mce_interface_, _mce_req_device_mode_change_, _STRING_, &_mce_device_FLIGHT_, _END_);
+	}
 	#endif
 }
 
@@ -582,17 +602,109 @@ static const char *_bme_reporting_design_ = "battery.reporting.design"; // metho
 static const char *_bme_reporting_current_ = "battery.reporting.current"; // method property
 static const char *_bme_remaining_time_ = "battery.remaining_time"; // method property
 
+static double last_battery_history_refresh;
+
+static void bme_update_drain( ){
+	if(hw_battery.history_head == hw_battery.history_tail){
+		// only zero or one data points
+		hw_battery.hours1 = 0; hw_battery.drain1 = 0.0;
+		hw_battery.hours24 = 2; hw_battery.drain24 = 0.0;
+		return;
+	}
+
+	double timestamp = hw_battery.history_tail->timestamp;
+	float capacity = hw_battery.history_tail->capacity;
+	float current_capacity = capacity;
+
+	// last hour
+	s_battery_history *history;
+	for(history = hw_battery.history_tail; history != NULL; history = history->prev){
+		timestamp = history->timestamp;
+		capacity = history->capacity;
+		if(history == hw_battery.charge_point || timestamp < current_time - 3600.0) break; // stop at the charging point or 1 hour ago
+	}
+	hw_battery.hours1 = (current_time - timestamp >= 3600.0 ? 1 : 0); // display tilde if less than 60 minutes of data
+	hw_battery.drain1 = (current_time > timestamp + 60.0 ? // wait 1 minute
+		roundf((capacity - current_capacity) * 36000.0 / (current_time - timestamp))/10.0
+		: 0.0);
+
+	// last 24 hours
+	if(hw_battery.charge_point){
+		timestamp = hw_battery.charge_point->timestamp;
+		capacity = hw_battery.charge_point->capacity;
+	} else {
+		timestamp = hw_battery.history_head->timestamp;
+		capacity = hw_battery.history_head->capacity;
+	}
+	if(current_time > timestamp + 5401.0){ // wait 1.5 hours
+		float hours = (current_time - timestamp) / 3600.0;
+		if(hours > 24.0) hours = 24.0;
+		hw_battery.hours24 = roundf(hours);
+		hw_battery.drain24 = roundf(10.0 * (capacity - current_capacity) / hours)/10.0;
+	} else {
+		hw_battery.hours24 = 2;
+		hw_battery.drain24 = 0.0;
+	}
+
+	hw_changes |= HW_BATTERY;
+	last_battery_history_refresh = current_time;
+}
+
 static void bme_get_capacity_callback( DBusPendingCall *pending, void *user_data ){
 	DBusMessage *message = dbus_pending_call_steal_reply(pending);
 	dbus_pending_call_unref(pending);
 	int current;
 	debug_write("[DBUS] callback: bme_get_capacity_callback");
 	if(dbus_message_get_args(message, &dbus_error, _INT32_, &current, _END_)){
-		float capacity = round(1000.0 * current / battery_reporting_design) / 10;
+		float capacity = roundf(1000.0 * current / battery_reporting_design) / 10;
+		unsigned update = 0;
 		if(capacity > 100.0) capacity = 100.0;
-		if(hw_battery.capacity != capacity){ hw_battery.capacity = capacity; hw_changes |= HW_BATTERY; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+		if(hw_battery.capacity != capacity){
+			hw_battery.capacity = capacity;
+			debug_write("battery capacity = %.2f%%", hw_battery.capacity);
+			test_write("battery capacity = %.2f%%", hw_battery.capacity);
+			dbus_send_battery_capacity_signal();
+			update = 1;
+
+			if(cfg_history_days){
+// TODO: this could test for a head element that will be purged below and reuse its memory instead of allocating here and freeing below
+				s_battery_history *history = (s_battery_history *)malloc(sizeof(s_battery_history));
+				if(history){
+					history->timestamp = current_time;
+					history->capacity = capacity;
+					history->prev = hw_battery.history_tail;
+					history->next = NULL;
+
+					if(hw_battery.history_head == NULL)
+						hw_battery.history_head = history;
+					else {
+						hw_battery.history_tail->next = history;
+						if(hw_battery.history_tail->capacity < capacity) hw_battery.charge_point = history; // set a charging point
+					}
+					hw_battery.history_tail = history;
+				} else
+					debug_write("[DBUS] bme_get_capacity_callback: Unable to allocate memory to store battery history data point");
+			}
+		}
+		if(cfg_history_days){
+			double base_time = current_time - cfg_history_days*86400;
+			s_battery_history *history;
+			for(history = hw_battery.history_head; history != NULL;){
+				s_battery_history *next = history->next;
+				if(history->timestamp > base_time) break;
+				// history element is older than storable time, remove it
+				if(hw_battery.history_head == history) hw_battery.history_head = next;
+				if(next == NULL) hw_battery.history_tail = NULL;
+				else next->prev = NULL;
+				if(history == hw_battery.charge_point) hw_battery.charge_point = NULL; // unset charging point
+				free(history);
+				update = 1;
+				history = next;
+			}
+			if(update) bme_update_drain();
+		}
+		if(update && is_value_visible(battery_widget_page)){ hw_changes |= HW_BATTERY; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 		hw_initialized &= ~HWIS_BME_CAPACITY;
-		dbus_send_battery_capacity_signal();
 	}
 	dbus_message_unref(message);
 }
@@ -617,7 +729,10 @@ static void bme_get_active_callback( DBusPendingCall *pending, void *user_data )
 	debug_write("[DBUS] callback: bme_get_active_callback");
 	if(dbus_message_get_args(message, &dbus_error, _INT32_, &active_seconds, _END_)){
 		unsigned active_minutes = active_seconds / 60;
-		if(hw_battery.active_minutes != active_minutes){ hw_battery.active_minutes = active_minutes; hw_changes |= HW_BATTERY; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+		if(hw_battery.active_minutes != active_minutes){
+			hw_battery.active_minutes = active_minutes;
+			if(is_value_visible(battery_widget_page)){ hw_changes |= HW_BATTERY; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+		}
 		hw_initialized &= ~HWIS_BME_ACTIVE;
 	}
 	dbus_message_unref(message);
@@ -631,8 +746,14 @@ static void bme_init( ){
 	hw_battery.capacity = 0.0;
 	hw_battery.active_minutes = 0;
 	hw_battery.idle_minutes = 0;
+	hw_battery.drain1 = hw_battery.drain24 = 0.0;
+	hw_battery.hours1 = 0;
+	hw_battery.hours24 = 2;
+	hw_battery.history_head = hw_battery.history_tail = hw_battery.charge_point = NULL;
 
 	#ifdef ARCH_armel
+	last_battery_history_refresh = 0.0;
+
 	hw_initialized |= HWIS_BME_CHARGER|HWIS_BME_MAX|HWIS_BME_CAPACITY|HWIS_BME_ACTIVE;
 	dbus_send_signal(_bme_request_path_, _bme_request_interface_, "status_info_req", _END_); // request charging status signal
 	hw_query_battery(0); // flushed by hw_init()
@@ -688,7 +809,10 @@ static void cpu_get_governor_callback( DBusPendingCall *pending, void *user_data
 		s_hw_governor *g;
 		for(g = hw_cpu.governor_list; g != NULL; g = g->next)
 			if(!strcmp(g->name, governor)) break;
-		if(hw_cpu.current_governor != g){ hw_cpu.current_governor = g; hw_changes |= HW_CPU; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+		if(hw_cpu.current_governor != g){
+			hw_cpu.current_governor = g;
+			if(is_value_visible(battery_widget_page)){ hw_changes |= HW_CPU; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+		}
 		// don't free the string
 	}
 	dbus_message_unref(message);
@@ -891,8 +1015,10 @@ static void hw_query_cpu_usage( ){
 		fscanf(in, "cpu  %u %u %u %*u %u %u %u", &user, &nice, &sys, /* idle, */ &iowait, &irq, &softirq);
 		fclose(in);
 		total = user + nice + sys + iowait + irq + softirq;
-		if(ticks)
-			usage = round(100.0*(float)(total - ticks) / (float)(ticks_per_second * interval));
+		if(ticks){
+			usage = roundf(100.0*(float)(total - ticks) / (float)(ticks_per_second * interval));
+			if(usage > 100) usage = 100;
+		}
 
 // TODO: this value would need to be divided by the number of cores, if more than one
 
@@ -949,9 +1075,9 @@ static void bluez_get_mode_callback( DBusPendingCall *pending, void *user_data )
 		if(hw_bluetooth.enabled != enabled || hw_bluetooth.visible != visible){
 			hw_bluetooth.enabled = enabled;
 			hw_bluetooth.visible = visible;
-			hw_changes |= HW_BLUETOOTH;
-			dbus_send_wakeup(_WAKEUP_REFRESH_);
+			if(is_value_visible(bluetooth_widget_page)){ hw_changes |= HW_BLUETOOTH; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 		}
+		hw_new_event("bluetooth", &hw_bluetooth.history_head, &hw_bluetooth.history_tail, hw_bluetooth.enabled);
 		hw_initialized &= ~HWIS_BLUETOOTH_MODE;
 	}
 	dbus_message_unref(message);
@@ -965,8 +1091,7 @@ static void bluez_get_name_callback( DBusPendingCall *pending, void *user_data )
 	if(dbus_message_get_args(message, &dbus_error, _STRING_, &name, _END_)){
 		if(hw_bluetooth.name == NULL || strcmp(hw_bluetooth.name, name)){
 			set_string(&hw_bluetooth.name, name, BLUETOOTH_NAME_LIMIT);
-			hw_changes |= HW_BLUETOOTH;
-			dbus_send_wakeup(_WAKEUP_REFRESH_);
+			if(is_value_visible(bluetooth_widget_page)){ hw_changes |= HW_BLUETOOTH; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 		}
 		hw_initialized &= ~HWIS_BLUETOOTH_NAME;
 	}
@@ -1018,8 +1143,7 @@ static void bluez_get_headset_state_callback( DBusPendingCall *pending, void *us
 		unsigned headset = (strcmp(state, "Disconnected") ? 1 : 0);
 		if(hw_bluetooth.headset != headset){
 			hw_bluetooth.headset = headset;
-			hw_changes |= HW_VOLUME_OUTPUT;
-			dbus_send_wakeup(_WAKEUP_REFRESH_);
+			if(is_value_visible(volume_widget_page)){ hw_changes |= HW_VOLUME_OUTPUT; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 		}
 		hw_initialized &= ~HWIS_BLUETOOTH_HEADSET;
 	}
@@ -1036,7 +1160,10 @@ static void bluez_get_headset_path_callback( DBusPendingCall *pending, void *use
 			dbus_recv_method(bluez_get_headset_state_callback, _bluez_audio_proxy_service_, path, _dbus_property_interface_, _dbus_Get_,
 								_STRING_, &empty, _STRING_, &_dbus_State_, _END_); // headset state
 		} else {
-			if(hw_bluetooth.headset){ hw_bluetooth.headset = 0; hw_changes |= HW_BLUETOOTH; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+			if(hw_bluetooth.headset){
+				hw_bluetooth.headset = 0;
+				if(is_value_visible(bluetooth_widget_page)){ hw_changes |= HW_BLUETOOTH; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+			}
 			hw_initialized &= ~HWIS_BLUETOOTH_HEADSET;
 		}
 	}
@@ -1051,6 +1178,8 @@ static void bluetooth_init( ){
 	hw_bluetooth.nr_devices = 0;
 	hw_bluetooth.headset = 0;
 
+	hw_bluetooth.history_head = hw_bluetooth.history_tail = NULL;
+
 	if(!svc_bluetooth_HCI) return;
 
 	#ifdef ARCH_armel
@@ -1158,7 +1287,11 @@ static void icd2_state_req_callback( DBusPendingCall *pending, void *user_data )
 	unsigned enabled;
 	debug_write("[DBUS] callback: icd2_state_req_callback");
 	if(dbus_message_get_args(message, &dbus_error, _UINT32_, &enabled, _END_)){
-		if(hw_wifi.enabled != enabled){ hw_wifi.enabled = enabled; hw_changes |= HW_WIFI; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+		if(hw_wifi.enabled != enabled){
+			hw_wifi.enabled = enabled;
+			if(is_value_visible(wifi_widget_page)){ hw_changes |= HW_WIFI; dbus_send_wakeup(_WAKEUP_REFRESH_); }
+		}
+		hw_new_event("wifi", &hw_wifi.history_head, &hw_wifi.history_tail, hw_wifi.enabled);
 		hw_initialized &= ~HWIS_WIFI_MODE;
 	}
 	dbus_message_unref(message);
@@ -1172,8 +1305,7 @@ static void icd_get_ipinfo_callback( DBusPendingCall *pending, void *user_data )
 	if(dbus_message_get_args(message, &dbus_error, _STRING_, &name, _STRING_, &addr, _STRING_, &mask, _STRING_, &gateway, _STRING_, &dns1, _STRING_, &dns2, _END_)){
 		if((hw_wifi.address ? strcmp(hw_wifi.address, addr) : 1)){
 			set_string(&hw_wifi.address, addr, WIFI_ADDRESS_LIMIT);
-			hw_changes |= HW_WIFI;
-			dbus_send_wakeup(_WAKEUP_REFRESH_);
+			if(is_value_visible(wifi_widget_page)){ hw_changes |= HW_WIFI; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 		}
 	}
 	dbus_message_unref(message);
@@ -1186,6 +1318,8 @@ static void wifi_init( ){
 	hw_wifi.address = NULL;
 	hw_wifi.state = WIFI_DISCONNECTED;
 
+	hw_wifi.history_head = hw_wifi.history_tail = NULL;
+
 	if(!svc_wifi_ICD) return;
 
 	#ifdef ARCH_armel
@@ -1315,8 +1449,7 @@ void hw_query_usb( ){
 	if(hw_usb.mode != mode || hw_usb.connected != connected){
 		hw_usb.mode = mode;
 		hw_usb.connected = connected;
-		hw_changes |= HW_USB;
-		dbus_send_wakeup(_WAKEUP_REFRESH_);
+		if(is_value_visible(usb_widget_page)){ hw_changes |= HW_USB; dbus_send_wakeup(_WAKEUP_REFRESH_); }
 	}
 
 	// this code assumes a_host/peripheral and b_host/peripheral means connected
@@ -1387,14 +1520,10 @@ void hw_refresh( ){
 		if(volume_widget_page == primary_widgets)
 			hw_query_volume();
 
-		if(battery_widget_page == primary_widgets)
+		if(battery_widget_page == primary_widgets){
 			hw_query_memory();
-
-		if(flightmode_button_page == primary_widgets){
-			// query flight mode status
-			hw_query_flight_mode(0);
 			#ifdef ARCH_armel
-			dbus_flush_messages();
+			if(cfg_history_days && current_time > last_battery_history_refresh + 60.0) bme_update_drain(); // 1 minute query interval
 			#endif
 		}
 
@@ -1429,6 +1558,8 @@ void hw_refresh( ){
 	#ifdef ARCH_armel
 	// poll initial values until all have been received
 	if(hw_initialized){
+		debug_write("acquiring initial [%u] values...", hw_initialized);
+
 		// MCE
 		if(hw_initialized & HWIS_MCE_DISPLAY)
 			dbus_recv_method(mce_get_display_callback, _mce_service_, _mce_path_, _mce_interface_, _mce_get_display_status_, _END_);
