@@ -7,6 +7,12 @@ typedef struct s_hw_swap {
 	unsigned total;
 } s_hw_swap;
 
+typedef struct s_event_history {
+	double timestamp;
+	short unsigned state;
+	struct s_event_history *next;
+} s_event_history;
+
 typedef struct s_hw_screen {
 	unsigned visible;
 	unsigned dimmed;
@@ -16,15 +22,27 @@ typedef struct s_hw_screen {
 	unsigned brightness_level;		// brightness level set by user
 	unsigned brightness_current;	// current brightness level
 	unsigned brightness_mismatch;	// current might not be same as level, keep checking with timer
+	s_event_history *history_head, *history_tail;
 } s_hw_screen;
 
 typedef enum e_battery_status { BATTERY_DRAINING, BATTERY_CHARGING, BATTERY_AC } e_battery_status;
+typedef struct s_battery_history {
+	double timestamp;
+	float capacity;
+	struct s_battery_history *prev, *next;
+} s_battery_history;
 typedef struct s_hw_battery {
 	unsigned initialize;
 	e_battery_status status;
 	float capacity;
 	unsigned active_minutes;
 	unsigned idle_minutes;
+	float drain1;
+	float drain24;
+	unsigned hours1;
+	unsigned hours24;
+	s_battery_history *history_head, *history_tail;
+	s_battery_history *charge_point;
 } s_hw_battery;
 
 typedef struct s_hw_governor {
@@ -54,20 +72,23 @@ typedef struct s_hw_bluetooth {
 	char *name;
 	unsigned nr_devices;	// number of connected devices
 	unsigned headset;		// headset connected
+	s_event_history *history_head, *history_tail;
 } s_hw_bluetooth;
 
 typedef struct s_hw_gps {
 	unsigned enabled;
+	s_event_history *history_head, *history_tail;
 } s_hw_gps;
 
 #define WIFI_NAME_LIMIT 25
 #define WIFI_ADDRESS_LIMIT 15
-typedef enum e_hw_wifi_state { WIFI_DISCONNECTED, WIFI_CONNECTING, WIFI_CONNECTED } e_hw_wifi_state;
+typedef enum e_hw_wifi_state { WIFI_DISCONNECTED = 0, WIFI_CONNECTING, WIFI_CONNECTED, WIFI_DISCONNECTING } e_hw_wifi_state;
 typedef struct s_hw_wifi {
 	unsigned enabled;
 	char *name;
 	char *address;
 	e_hw_wifi_state state;
+	s_event_history *history_head, *history_tail;
 } s_hw_wifi;
 
 typedef struct s_hw_sensors {
@@ -123,7 +144,7 @@ extern int hw_volume;
 extern s_hw_memory hw_memory;
 extern s_hw_swap hw_swap;
 extern s_hw_screen hw_screen;
-extern unsigned hw_flight_mode;
+extern unsigned hw_flight_mode, hw_entering_flight_mode;
 extern unsigned hw_device_locked;
 extern s_hw_battery hw_battery;
 extern s_hw_cpu hw_cpu;
@@ -139,6 +160,7 @@ extern double last_hw_refresh;
 //////////////////////////////////////////////////////////////////////////
 
 int get_file_value( const char *filename );
+void hw_new_event( const char *label, s_event_history **head, s_event_history **tail, unsigned state );
 
 void brightness_init( );
 void hw_query_brightness( );
