@@ -100,7 +100,7 @@ unsigned (*motion_callback)( );
 
 unsigned primary_widgets, widget_edit_mode;
 s_selected_cell selected_cell;
-unsigned brightness_widget_page, volume_widget_page, battery_widget_page;
+unsigned brightness_widget_page, volume_widget_page, battery_widget_page, wifi_widget_page, bluetooth_widget_page, usb_widget_page;
 unsigned flightmode_button_page, keyrepeat_button_page, screenstayslit_button_page, screenautolock_button_page;
 		// new *_page variables must be added to window_map() and dialog_reset()
 
@@ -458,6 +458,9 @@ void error_write( const char *format, ... ){
 	#ifdef DEBUG
 	debug_write("ERROR: %s", buffer);
 	#endif
+	#ifdef TESTLOG
+	test_write("ERROR: %s", buffer);
+	#endif
 }
 #endif
 
@@ -1487,7 +1490,7 @@ if(runlevel == 2){
 
 					// ignore the tap used to wake screen
 					click.enabled = 1;
-					notify_screen_unlocked();
+					if(cfg_tkactive_notifications) notify_screen_unlocked();
 
 				}
 
@@ -1560,7 +1563,7 @@ if(runlevel == 2){
 
 					// ignore the key used to wake screen
 					click.enabled = 1;
-					notify_screen_unlocked();
+					if(cfg_tkactive_notifications) notify_screen_unlocked();
 
 				}
 
