@@ -75,7 +75,7 @@ typedef struct s_selected_cell {
 	unsigned subcell;
 } s_selected_cell;
 extern s_selected_cell selected_cell;
-extern unsigned brightness_widget_page, volume_widget_page, battery_widget_page;
+extern unsigned brightness_widget_page, volume_widget_page, battery_widget_page, wifi_widget_page, bluetooth_widget_page, usb_widget_page;
 extern unsigned flightmode_button_page, keyrepeat_button_page, screenstayslit_button_page, screenautolock_button_page;
 
 extern unsigned power_unlock_keys[];
