@@ -57,6 +57,8 @@ const char *conf_battery_idle_time = "/apps/asui/battery_idle_time";
 const char *conf_memory_format = "/apps/asui/memory_format";
 const char *conf_swap_format = "/apps/asui/swap_format";
 const char *conf_temperature_F = "/apps/asui/temperature_F";
+const char *conf_history_days = "/apps/asui/history_days";
+const char *conf_track_usage_events = "/apps/asui/track_usage_events";
 const char *conf_enable_battery_applet = "/apps/asui/enable_battery_applet";
 // clock
 const char *conf_clock_24h = "/apps/asui/clock_24h";
@@ -91,6 +93,7 @@ const char *conf_secure_power_action = "/apps/asui/secure_power_action";
 const char *conf_secure_button = "/apps/asui/secure_button";
 const char *conf_power_unlock_key = "/apps/asui/power_unlock_key";
 // user interface
+const char *conf_tkactive_notifications = "/apps/asui/tkactive_notifications";
 const char *conf_service_size_unit = "/apps/asui/service_size_unit";
 const char *conf_short_tap_markers = "/apps/asui/short_tap_markers";
 const char *conf_start_visible = "/apps/asui/start_visible";
@@ -206,16 +209,26 @@ GtkWidget *new_button_right( GtkWidget *parent, char *name, void (*callback)(Gtk
 	return button;
 }
 
-GtkWidget *new_toggle_button( GtkWidget *parent, char *name, void (*callback)(GtkWidget *widget, gpointer data), gboolean state, gboolean expand, gboolean fill ){
+GtkWidget *new_toggle_button( GtkWidget *parent, char *name, void (*callback)(GtkWidget *widget, gpointer data), void *data, gboolean state, gboolean expand, gboolean fill ){
 	GtkWidget *button = gtk_toggle_button_new_with_label(name);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), state);
-	gtk_signal_connect(GTK_OBJECT(button), "toggled", GTK_SIGNAL_FUNC(callback), NULL);
+	gtk_signal_connect(GTK_OBJECT(button), "toggled", GTK_SIGNAL_FUNC(callback), data);
 	gtk_widget_set_size_request(button, -1, 40);
 	gtk_box_pack_start(GTK_BOX(parent), button, expand, fill, 0);
 	gtk_widget_show(button);
 	return button;
 }
 
+GtkWidget *new_toggle_button_right( GtkWidget *parent, char *name, void (*callback)(GtkWidget *widget, gpointer data), void *data, gboolean state ){
+	GtkWidget *button = gtk_toggle_button_new_with_label(name);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), state);
+	gtk_signal_connect(GTK_OBJECT(button), "toggled", GTK_SIGNAL_FUNC(callback), data);
+	gtk_widget_set_size_request(button, -1, 40);
+	gtk_box_pack_end(GTK_BOX(parent), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
+	return button;
+}
+
 ////////////////////
 
 static void default_checkbox_callback( GtkToggleButton *widget, gpointer data ){
