@@ -27,6 +27,8 @@ extern const char *conf_battery_idle_time;
 extern const char *conf_memory_format;
 extern const char *conf_swap_format;
 extern const char *conf_temperature_F;
+extern const char *conf_history_days;
+extern const char *conf_track_usage_events;
 extern const char *conf_enable_battery_applet;
 // clock
 extern const char *conf_clock_24h;
@@ -61,6 +63,7 @@ extern const char *conf_secure_power_action;
 extern const char *conf_secure_button;
 extern const char *conf_power_unlock_key;
 // user interface
+extern const char *conf_tkactive_notifications;
 extern const char *conf_service_size_unit;
 extern const char *conf_short_tap_markers;
 extern const char *conf_start_visible;
@@ -80,7 +83,8 @@ void new_section_vbox( char *name );
 GtkWidget *new_message( char *message );
 GtkWidget *new_button( GtkWidget *parent, char *name, void (*callback)(GtkWidget *widget, gpointer data), void *dta );
 GtkWidget *new_button_right( GtkWidget *parent, char *name, void (*callback)(GtkWidget *widget, gpointer data), void *data );
-GtkWidget *new_toggle_button( GtkWidget *parent, char *name, void (*callback)(GtkWidget *widget, gpointer data), gboolean state, gboolean expand, gboolean fill );
+GtkWidget *new_toggle_button( GtkWidget *parent, char *name, void (*callback)(GtkWidget *widget, gpointer data), void *data, gboolean state, gboolean expand, gboolean fill );
+GtkWidget *new_toggle_button_right( GtkWidget *parent, char *name, void (*callback)(GtkWidget *widget, gpointer data), void *data, gboolean state );
 GtkWidget *new_checkbox( const char *conf, char *label, void (*callback)(GtkToggleButton *widget, gpointer data) );
 GtkWidget *new_selector( const char *conf, char *name, const char **items, void (*callback)(GtkComboBox *widget, gpointer data) );
 GtkWidget *new_uint_value( const char *conf, char *name, unsigned min_value, unsigned max_value, void (*callback)(GObject *gobject, GParamSpec *arg1, gpointer user_data) );
